Package: hmer
Type: Package
Title: History Matching and Emulation Package
Version: 1.6.0
Authors@R: c(person("Andrew", "Iskauskas", email = "andrew.iskauskas@durham.ac.uk", comment = c(ORCID = "0000-0003-2825-3651"), role = c('aut', 'cre')),
  person("TJ", "McKinley", email = "t.mckinley@exeter.ac.uk", comment = c(ORCID = "0000-0002-9485-3236"), role = c('aut')))
Maintainer: Andrew Iskauskas <andrew.iskauskas@durham.ac.uk>
BugReports: https://github.com/andy-iskauskas/hmer/issues
URL: https://github.com/andy-iskauskas/hmer,
        https://hmer-package.github.io/website/
Description: A set of objects and functions for Bayes Linear emulation and history matching.
  Core functionality includes automated training of emulators to data, diagnostic functions
  to ensure suitability, and a variety of proposal methods for generating 'waves' of points.
  For details on the mathematical background, there are many papers available on the topic
  (see references attached to function help files or the below references); for details of 
  the functions in this package, consult the manual or help files.
  Iskauskas, A, et al. (2024) <doi:10.18637/jss.v109.i10>.
  Bower, R.G., Goldstein, M., and Vernon, I. (2010) <doi:10.1214/10-BA524>.
  Craig, P.S., Goldstein, M., Seheult, A.H., and Smith, J.A. (1997) <doi:10.1007/978-1-4612-2290-3_2>.
Suggests: spelling, knitr, rmarkdown, deSolve, testthat (>= 3.0.0),
        covr, progressr
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: purrr, dplyr, ggplot2, lhs, MASS, R6, viridis, mvtnorm,
        GGally, rlang, isoband, cluster, pdist, ggbeeswarm, stringr
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.1
Language: en-GB
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-05-31 08:39:53 UTC; tqxh68
Author: Andrew Iskauskas [aut, cre] (<https://orcid.org/0000-0003-2825-3651>),
  TJ McKinley [aut] (<https://orcid.org/0000-0002-9485-3236>)
Repository: CRAN
Date/Publication: 2024-05-31 13:30:07 UTC
Built: R 4.4.0; ; 2024-06-01 16:10:30 UTC; unix
