# Generated by roxygen2: do not edit by hand

export(Correlator)
export(Emulator)
export(HierarchicalEmulator)
export(Proto_emulator)
export(analyze_diagnostic)
export(behaviour_plot)
export(bimodal_emulator_from_data)
export(classification_diag)
export(collect_emulators)
export(comparison_diag)
export(diagnostic_pass)
export(diagnostic_wrap)
export(directional_deriv)
export(directional_proposal)
export(effect_strength)
export(emulator_from_data)
export(emulator_plot)
export(exp_sq)
export(full_wave)
export(gamma_exp)
export(generate_new_design)
export(generate_new_runs)
export(get_diagnostic)
export(hit_by_wave)
export(idemc)
export(individual_errors)
export(matern)
export(nth_implausible)
export(orn_uhl)
export(output_plot)
export(plot_actives)
export(plot_lattice)
export(plot_wrap)
export(rat_quad)
export(residual_diag)
export(simulator_plot)
export(space_removal)
export(space_removed)
export(standard_errors)
export(subset_emulators)
export(summary_diag)
export(validation_diagnostics)
export(validation_pairs)
export(variance_emulator_from_data)
export(wave_dependencies)
export(wave_points)
export(wave_values)
import(ggplot2)
importFrom(GGally,ggally_blank)
importFrom(GGally,ggally_densityDiag)
importFrom(GGally,ggally_text)
importFrom(GGally,ggmatrix)
importFrom(GGally,ggpairs)
importFrom(GGally,grab_legend)
importFrom(GGally,putPlot)
importFrom(GGally,wrap)
importFrom(MASS,ginv)
importFrom(R6,R6Class)
importFrom(cluster,daisy)
importFrom(cluster,fanny)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,group_by)
importFrom(dplyr,group_rows)
importFrom(dplyr,mutate)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(lhs,randomLHS)
importFrom(mvtnorm,dmvnorm)
importFrom(mvtnorm,rmvnorm)
importFrom(pdist,pdist)
importFrom(purrr,exec)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map2_dbl)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_lgl)
importFrom(rlang,as_string)
importFrom(rlang,hash)
importFrom(rlang,quo_get_expr)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,reshape)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,step)
importFrom(stats,var)
importFrom(stats,xtabs)
importFrom(stringr,str_pad)
importFrom(utils,stack)
importFrom(utils,write.csv)
importFrom(viridis,scale_colour_viridis)
importFrom(viridis,scale_fill_viridis)
importFrom(viridis,viridis)
